==> emelfm2-0.1.2.tar.gz: emelfm2-0.1.2/src/utils/cell-renderers/mg-cell-renderer-list.c <==
/*  $Id: mg-cell-renderer-list.c 1003 2005-08-02 03:58:09Z tpgww $
 *
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * Copyright (C) 2001-2002 CodeFactory AB
 * Copyright (C) 2001-2002 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2001-2002 Mikael Hallendal <micke@codefactory.se>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_tree.h"
#include "mg-cell-renderer-list.h"
#include "mg-popup-entry.h"

static void mcrl_init (MgCellRendererList *cell);
static void mcrl_class_init (MgCellRendererListClass *class);
static GtkCellEditable * mcrp_start_editing (GtkCellRenderer *cell,
	GdkEvent *event, GtkWidget *widget, const gchar *path, GdkRectangle *b,
	GdkRectangle *c, GtkCellRendererState flags);

static void     mcrl_show_popup           (MgCellRendererPopup     *cell,
					   const gchar             *path,
					   gint                     x1,
					   gint                     y1,
					   gint                     x2,
					   gint                     y2);
static void     mcrl_hide_popup           (MgCellRendererPopup     *cell);

static gboolean mcrl_button_press_event   (GtkWidget               *widget,
					   GdkEventButton          *event,
					   MgCellRendererList      *list);


static MgCellRendererPopupClass *parent_class;


GType
mg_cell_renderer_list_get_type (void)
{
	static GType type = 0;
	
	if (!type) {
		static const GTypeInfo info = {
			sizeof (MgCellRendererListClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) mcrl_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (MgCellRendererList),
			0,              /* n_preallocs */
			(GInstanceInitFunc) mcrl_init,
		};
		
		type = g_type_register_static (MG_TYPE_CELL_RENDERER_POPUP,
					       "MgCellRendererList",
					       &info,
					       0);
	}
	
	return type;
}

gboolean selection_check_cb (GtkTreeSelection *selection, GtkTreeModel *model,
	GtkTreePath *path, gboolean path_currently_selected, gpointer data)
{
	gboolean select;
	GtkTreeIter iter;
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gtk_tree_model_get (model, &iter, 3, &select, -1);
		return select;
	}
	return TRUE;
}

static void mcrl_init (MgCellRendererList *cell)
{
	MgCellRendererPopup *popup_cell;
	GtkCellRenderer     *textcell;

	popup_cell = MG_CELL_RENDERER_POPUP (cell);
	
	cell->scrolled = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (popup_cell->popup_window), cell->scrolled);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (cell->scrolled),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (cell->scrolled),
		GTK_SHADOW_OUT);
	gtk_widget_show (cell->scrolled);

	cell->tree_view = gtk_tree_view_new ();
	gtk_container_add (GTK_CONTAINER (cell->scrolled), cell->tree_view);
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (cell->tree_view), FALSE);
	textcell = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cell->tree_view),
		0, NULL, textcell, "text", 1, NULL);

	g_signal_connect (cell->tree_view, "button-release-event",
		G_CALLBACK (mcrl_button_press_event), cell);


	cell->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (cell->tree_view));
	gtk_tree_selection_set_mode (cell->selection, GTK_SELECTION_BROWSE);
	gtk_tree_selection_set_select_function (cell->selection,
		(GtkTreeSelectionFunc) selection_check_cb, NULL, NULL);
	gtk_widget_show (cell->tree_view);

	popup_cell->focus_window = cell->tree_view;
}

static void mcrl_class_init (MgCellRendererListClass *class)
{
	MgCellRendererPopupClass *popup_class;

	popup_class = MG_CELL_RENDERER_POPUP_CLASS (class);

	parent_class = MG_CELL_RENDERER_POPUP_CLASS (
		g_type_class_peek_parent (class));

	GTK_CELL_RENDERER_CLASS (popup_class)->start_editing = mcrp_start_editing;
	popup_class->show_popup = mcrl_show_popup;
	popup_class->hide_popup = mcrl_hide_popup;
}

static GtkCellEditable *mcrp_start_editing (GtkCellRenderer *cell,
	GdkEvent *event, GtkWidget *widget, const gchar *path, GdkRectangle *b,
	GdkRectangle *c, GtkCellRendererState flags)
{
	GtkCellEditable *edit = GTK_CELL_RENDERER_CLASS (parent_class)->start_editing
		(cell, event, widget, path, b, c, flags);
	gint x, y, m, n;
	
	//FIXME: bad menu position when activated through key press
	if (event == NULL)
	{
		e2_utils_get_abs_pos (widget, &x, &y);
		y += c->height + (2 * widget->style->ythickness) + 4;
		//FIXME: bad hack for the cell renderer to be able to calculate
		//correct values if activated through a key-press
		GtkWidget *sw = g_object_get_data (G_OBJECT (cell), "scrolled-window");
		if (sw != NULL)
		{
			gint val = (gint) gtk_adjustment_get_value
				(gtk_scrolled_window_get_hadjustment GTK_SCROLLED_WINDOW (sw));
			x -= val;
		}
	} else
		gdk_window_get_origin (((GdkEventAny *) event)->window, &x, &y);
	
	m = x + c->x;
	n = y + c->y;
	g_signal_emit_by_name (G_OBJECT (cell), "show-popup", path,
		m, n, m + c->width, n + c->height);
	return GTK_CELL_EDITABLE (edit);
}
static void mcrl_show_popup (MgCellRendererPopup *cell, const gchar *path,
	gint x1, gint y1, gint x2, gint y2)
{
	MgCellRendererList *list_cell = MG_CELL_RENDERER_LIST (cell);
	
	if (list_cell->model == NULL)
		return;

	if (parent_class->show_popup)
	{
		gtk_tree_view_set_model (GTK_TREE_VIEW (list_cell->tree_view),
			GTK_TREE_MODEL (list_cell->model));
	
		gchar *text;
		g_object_get (G_OBJECT (cell), "text", &text, NULL);
		
		gtk_tree_view_expand_all (GTK_TREE_VIEW (list_cell->tree_view));
	
		GtkTreePath *path_sel;
		GtkTreeIter iter;
		if (e2_tree_find_iter_from_str (GTK_TREE_MODEL (list_cell->model), 0,
			text, &iter, FALSE))
			path_sel = gtk_tree_model_get_path (GTK_TREE_MODEL (list_cell->model), &iter);
		else
			path_sel = gtk_tree_path_new_from_string ("0");
		gtk_tree_view_set_cursor (GTK_TREE_VIEW (list_cell->tree_view), path_sel,
			NULL, FALSE);
		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (list_cell->tree_view), path_sel,
			NULL, FALSE, 0.5, 0.5);
		gtk_widget_set_size_request (list_cell->tree_view, 1, -1);
		GtkRequisition req;
		gtk_widget_size_request (list_cell->tree_view, &req);
		gint height = req.height;
		gint screen_height = gdk_screen_height ();
		if (height > (screen_height / 2))
		{
			height = screen_height / 2 - 50;
			gtk_widget_set_size_request (list_cell->tree_view, 1, height);
		}
	//	gtk_window_get_size (GTK_WINDOW ((list_cell->parent).popup_window), NULL, &height);
		gtk_window_resize (GTK_WINDOW ((list_cell->parent).popup_window), x2 - x1, height);
	
	//	if (parent_class->show_popup)
			parent_class->show_popup (cell, path, x1, y1, x2, y2);
	}
}

static void mcrl_hide_popup (MgCellRendererPopup *cell)
{
	MgCellRendererList *list_cell = MG_CELL_RENDERER_LIST (cell);
	if (parent_class->hide_popup)
		parent_class->hide_popup (cell);
	list_cell->model = NULL; 
}
	 
GtkCellRenderer *mg_cell_renderer_list_new (void)
{
	GObject *cell = g_object_new (MG_TYPE_CELL_RENDERER_LIST, NULL);
	return GTK_CELL_RENDERER (cell);
}

static gboolean mcrl_button_press_event (GtkWidget *widget,
	GdkEventButton *event, MgCellRendererList *list)
{
	if (event->button == 1)
	{
		GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW
			(list->tree_view));

		GtkTreePath *path;
		// Get tree path for row that was clicked
		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (list->tree_view),
				event->x, event->y, &path, NULL, NULL, NULL))
		{
			GtkTreeIter iter;
			gtk_tree_model_get_iter (GTK_TREE_MODEL (list->model), &iter, path);
			gboolean select;
			gtk_tree_model_get (GTK_TREE_MODEL (list->model), &iter, 3, &select, -1);
			if (select)
			{
				gtk_tree_selection_unselect_all (selection);
				gtk_tree_selection_select_path (selection, path);
				mg_cell_renderer_popup_hide (MG_CELL_RENDERER_POPUP (list));
				gtk_tree_path_free (path);
				return TRUE;
			}
			gtk_tree_path_free (path);
		}
		return FALSE;
	}
	return FALSE;
}

==> emelfm2-0.1.4.tar.gz: emelfm2-0.1.4/src/utils/cell-renderers/mg-cell-renderer-list.c <==
/*  $Id: mg-cell-renderer-list.c 1140 2005-09-16 01:44:56Z tpgww $
 *
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * Copyright (C) 2001-2002 CodeFactory AB
 * Copyright (C) 2001-2002 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2001-2002 Mikael Hallendal <micke@codefactory.se>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_tree.h"
#include "mg-cell-renderer-list.h"
#include "mg-popup-entry.h"

static void mcrl_init (MgCellRendererList *cell);
static void mcrl_class_init (MgCellRendererListClass *class);
static GtkCellEditable * mcrp_start_editing (GtkCellRenderer *cell,
	GdkEvent *event, GtkWidget *widget, const gchar *path, GdkRectangle *b,
	GdkRectangle *c, GtkCellRendererState flags);

static void     mcrl_show_popup           (MgCellRendererPopup     *cell,
					   const gchar             *path,
					   gint                     x1,
					   gint                     y1,
					   gint                     x2,
					   gint                     y2);
static void     mcrl_hide_popup           (MgCellRendererPopup     *cell);

static gboolean mcrl_button_press_event   (GtkWidget               *widget,
					   GdkEventButton          *event,
					   MgCellRendererList      *list);


static MgCellRendererPopupClass *parent_class;


GType
mg_cell_renderer_list_get_type (void)
{
	static GType type = 0;
	
	if (!type) {
		static const GTypeInfo info = {
			(guint16)sizeof (MgCellRendererListClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) mcrl_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			(guint16)sizeof (MgCellRendererList),
			(guint16)0,   /* n_preallocs */
			(GInstanceInitFunc) mcrl_init,
			NULL      /* value_table */
		};
		
		type = g_type_register_static (MG_TYPE_CELL_RENDERER_POPUP,
					       "MgCellRendererList",
					       &info,
					       0);
	}
	
	return type;
}

gboolean selection_check_cb (GtkTreeSelection *selection, GtkTreeModel *model,
	GtkTreePath *path, gboolean path_currently_selected, gpointer data)
{
	gboolean select;
	GtkTreeIter iter;
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gtk_tree_model_get (model, &iter, 3, &select, -1);
		return select;
	}
	return TRUE;
}

static void mcrl_init (MgCellRendererList *cell)
{
	MgCellRendererPopup *popup_cell;
	GtkCellRenderer     *textcell;

	popup_cell = MG_CELL_RENDERER_POPUP (cell);
	
	cell->scrolled = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (popup_cell->popup_window), cell->scrolled);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (cell->scrolled),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (cell->scrolled),
		GTK_SHADOW_OUT);
	gtk_widget_show (cell->scrolled);

	cell->tree_view = gtk_tree_view_new ();
	gtk_container_add (GTK_CONTAINER (cell->scrolled), cell->tree_view);
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (cell->tree_view), FALSE);
	textcell = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cell->tree_view),
		0, NULL, textcell, "text", 1, NULL);

	g_signal_connect (cell->tree_view, "button-release-event",
		G_CALLBACK (mcrl_button_press_event), cell);


	cell->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (cell->tree_view));
	gtk_tree_selection_set_mode (cell->selection, GTK_SELECTION_BROWSE);
	gtk_tree_selection_set_select_function (cell->selection,
		(GtkTreeSelectionFunc) selection_check_cb, NULL, NULL);
	gtk_widget_show (cell->tree_view);

	popup_cell->focus_window = cell->tree_view;
}

static void mcrl_class_init (MgCellRendererListClass *class)
{
	MgCellRendererPopupClass *popup_class;

	popup_class = MG_CELL_RENDERER_POPUP_CLASS (class);

	parent_class = MG_CELL_RENDERER_POPUP_CLASS (
		g_type_class_peek_parent (class));

	GTK_CELL_RENDERER_CLASS (popup_class)->start_editing = mcrp_start_editing;
	popup_class->show_popup = mcrl_show_popup;
	popup_class->hide_popup = mcrl_hide_popup;
}

static GtkCellEditable *mcrp_start_editing (GtkCellRenderer *cell,
	GdkEvent *event, GtkWidget *widget, const gchar *path, GdkRectangle *b,
	GdkRectangle *c, GtkCellRendererState flags)
{
	GtkCellEditable *edit = GTK_CELL_RENDERER_CLASS (parent_class)->start_editing
		(cell, event, widget, path, b, c, flags);
	gint x, y, m, n;
	
	//FIXME: bad menu position when activated through key press
	if (event == NULL)
	{
		e2_utils_get_abs_pos (widget, &x, &y);
		y += c->height + (2 * widget->style->ythickness) + 4;
		//FIXME: bad hack for the cell renderer to be able to calculate
		//correct values if activated through a key-press
		GtkWidget *sw = g_object_get_data (G_OBJECT (cell), "scrolled-window");
		if (sw != NULL)
		{
			gint val = (gint) gtk_adjustment_get_value
				(gtk_scrolled_window_get_hadjustment GTK_SCROLLED_WINDOW (sw));
			x -= val;
		}
	} else
		gdk_window_get_origin (((GdkEventAny *) event)->window, &x, &y);
	
	m = x + c->x;
	n = y + c->y;
	g_signal_emit_by_name (G_OBJECT (cell), "show-popup", path,
		m, n, m + c->width, n + c->height);
	return GTK_CELL_EDITABLE (edit);
}
static void mcrl_show_popup (MgCellRendererPopup *cell, const gchar *path,
	gint x1, gint y1, gint x2, gint y2)
{
	MgCellRendererList *list_cell = MG_CELL_RENDERER_LIST (cell);
	
	if (list_cell->model == NULL)
		return;

	if (parent_class->show_popup)
	{
		gtk_tree_view_set_model (GTK_TREE_VIEW (list_cell->tree_view),
			GTK_TREE_MODEL (list_cell->model));
	
		gchar *text;
		g_object_get (G_OBJECT (cell), "text", &text, NULL);
		
		gtk_tree_view_expand_all (GTK_TREE_VIEW (list_cell->tree_view));
	
		GtkTreePath *path_sel;
		GtkTreeIter iter;
		if (e2_tree_find_iter_from_str (GTK_TREE_MODEL (list_cell->model), 0,
			text, &iter, FALSE))
			path_sel = gtk_tree_model_get_path (GTK_TREE_MODEL (list_cell->model), &iter);
		else
			path_sel = gtk_tree_path_new_from_string ("0");
		gtk_tree_view_set_cursor (GTK_TREE_VIEW (list_cell->tree_view), path_sel,
			NULL, FALSE);
		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (list_cell->tree_view), path_sel,
			NULL, FALSE, 0.5, 0.5);
		gtk_widget_set_size_request (list_cell->tree_view, 1, -1);
		GtkRequisition req;
		gtk_widget_size_request (list_cell->tree_view, &req);
		gint height = req.height;
		gint screen_height = gdk_screen_height ();
		if (height > (screen_height / 2))
		{
			height = screen_height / 2 - 50;
			gtk_widget_set_size_request (list_cell->tree_view, 1, height);
		}
	//	gtk_window_get_size (GTK_WINDOW ((list_cell->parent).popup_window), NULL, &height);
		gtk_window_resize (GTK_WINDOW ((list_cell->parent).popup_window), x2 - x1, height);
	
	//	if (parent_class->show_popup)
			parent_class->show_popup (cell, path, x1, y1, x2, y2);
	}
}

static void mcrl_hide_popup (MgCellRendererPopup *cell)
{
	MgCellRendererList *list_cell = MG_CELL_RENDERER_LIST (cell);
	if (parent_class->hide_popup)
		parent_class->hide_popup (cell);
	list_cell->model = NULL; 
}
	 
GtkCellRenderer *mg_cell_renderer_list_new (void)
{
	GObject *cell = g_object_new (MG_TYPE_CELL_RENDERER_LIST, NULL);
	return GTK_CELL_RENDERER (cell);
}

static gboolean mcrl_button_press_event (GtkWidget *widget,
	GdkEventButton *event, MgCellRendererList *list)
{
	if (event->button == 1)
	{
		GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW
			(list->tree_view));

		GtkTreePath *path;
		// Get tree path for row that was clicked
		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (list->tree_view),
				event->x, event->y, &path, NULL, NULL, NULL))
		{
			GtkTreeIter iter;
			gtk_tree_model_get_iter (GTK_TREE_MODEL (list->model), &iter, path);
			gboolean select;
			gtk_tree_model_get (GTK_TREE_MODEL (list->model), &iter, 3, &select, -1);
			if (select)
			{
				gtk_tree_selection_unselect_all (selection);
				gtk_tree_selection_select_path (selection, path);
				mg_cell_renderer_popup_hide (MG_CELL_RENDERER_POPUP (list));
				gtk_tree_path_free (path);
				return TRUE;
			}
			gtk_tree_path_free (path);
		}
		return FALSE;
	}
	return FALSE;
}
